@@ -34,7 +34,6 @@ module ByteStringUtils (
         gzDecompress,
 
         -- list utilities
-        ifHeadThenTail,
         dropSpace,
         breakSpace,
         linesPS,
@@ -53,18 +52,17 @@ module ByteStringUtils (
         intercalate,
 
         -- encoding and unicode utilities
+        isAscii,
         decodeLocale,
         encodeLocale,
-        encodeLatin1,
-        decodeString,
-        utf8ToLocale
+        decodeString
     ) where
 
 import Prelude hiding ( catch )
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Char8      as BC
 import qualified Data.ByteString.Internal   as BI
-import Data.ByteString (intercalate, uncons)
+import Data.ByteString (intercalate)
 import Data.ByteString.Internal (fromForeignPtr)
 
 #if defined (HAVE_MMAP)
@@ -72,7 +70,7 @@ import Control.Exception.Extensible ( catch, SomeException )
 #endif
 import System.IO
 import System.IO.Unsafe         ( unsafePerformIO )
-import System.Console.Haskeline ( runInputT, defaultSettings )
+import System.Console.Haskeline ( InputT, runInputTBehavior, defaultSettings, useFileHandle )
 import System.Console.Haskeline.Encoding ( decode, encode )
 
 import Foreign.Storable         ( peekElemOff, peek )
@@ -140,15 +138,6 @@ unpackPSFromUTF8  = T.unpack . decodeUtf8With lenientDecode
 packStringToUTF8 :: String -> B.ByteString
 packStringToUTF8 = encodeUtf8 . T.pack
 
--- -----------------------------------------------------------------------------
--- List-mimicking functions for PackedStrings
-
-{-# INLINE ifHeadThenTail #-}
-ifHeadThenTail :: Word8 -> B.ByteString -> Maybe B.ByteString
-ifHeadThenTail c s = case uncons s of
-    Just (w, t) | w == c    -> Just t
-    _                       -> Nothing
-
 ------------------------------------------------------------------------
 -- A reimplementation of Data.ByteString.Char8.dropSpace, but
 -- specialised to darcs' need for a 4 way isspace.
@@ -238,7 +227,7 @@ hashPS ps =
    case BI.toForeignPtr ps of
    (x,s,l) ->
     unsafePerformIO $ withForeignPtr x $ \p->
-    do hash (p `plusPtr` s) l
+    hash (p `plusPtr` s) l
 
 hash :: Ptr Word8 -> Int -> IO Int32
 hash ptr len = f (0 :: Int32) ptr len
@@ -528,12 +517,21 @@ breakBeforeNthNewline n the_ps =
           end = the_s + l
       findit n the_s
 
+-- we use a custom Behavior because otherwise on Windows haskeline tries to
+-- do hGetEcho on stdin which fails if stdin is e.g. /dev/null
+unsafeRunInput :: InputT IO a -> a
+unsafeRunInput = unsafePerformIO . runInputTBehavior (useFileHandle stdin) defaultSettings
+
+-- | Test if a ByteString is made of ascii characters
+isAscii :: B.ByteString -> Bool
+isAscii = B.all (\w -> w < 128)
+
 -- | Decode a ByteString to a String according to the current locale
 -- unsafePerformIO in the locale function is ratified by the fact that GHC 6.12
 -- and above also supply locale conversion with functions with a pure type.
 -- Unrecognized byte sequences in the input are skipped.
 decodeLocale :: B.ByteString -> String
-decodeLocale = unsafePerformIO . runInputT defaultSettings . decode
+decodeLocale = unsafeRunInput . decode
 
 -- | Encode a String to a ByteString with latin1 (i.e., the values of the
 -- characters become the values of the bytes; if a character value is greater
@@ -543,15 +541,11 @@ encodeLatin1 = B.pack . (map (fromIntegral . ord))
 
 -- | Encode a String to a ByteString according to the current locale
 encodeLocale :: String -> B.ByteString
-encodeLocale = unsafePerformIO . runInputT defaultSettings . encode
+encodeLocale = unsafeRunInput . encode
 
 -- | Take a @String@ that represents byte values and re-decode it acording to
 -- the current locale.
 decodeString :: String -> String
 decodeString = decodeLocale . encodeLatin1
 
--- | Convert a bytestring representing a text from UTF-8 to the current locale
-utf8ToLocale :: B.ByteString -> B.ByteString
-utf8ToLocale bs = encodeLocale string
-  where string = unpackPSFromUTF8 bs
 
