@@ -2,18 +2,18 @@
 
 module Darcs.External (
     backupByRenaming, backupByCopying,
-    copyFileOrUrl, speculateFileOrUrl, copyFilesOrUrls, copyLocal, cloneFile,
-    cloneTree, cloneTreeExcept, clonePartialsTree, clonePaths,
+    copyFileOrUrl, speculateFileOrUrl, copyLocal, cloneFile,
+    cloneTree, cloneTreeExcept,
     fetchFilePS, fetchFileLazyPS, gzFetchFilePS,
     sendEmail, generateEmail, sendEmailDoc, resendEmail,
     signString, verifyPS,
     execDocPipe, execPipeIgnoreError,
     getTermNColors,
     pipeDoc, pipeDocSSH, execSSH,
-    remoteDarcsCmd,
     maybeURLCmd,
     Cachable(Cachable, Uncachable, MaxAge),
     viewDoc, viewDocWith,
+    haveSendmail,
     sendmailPath, diffProgram, darcsProgram
   ) where
 
@@ -47,14 +47,14 @@ import Darcs.Lock ( canonFilename, writeDocBinFile )
 import System.Console.Terminfo( tiGetNum, setupTermFromEnv, getCapability )
 #endif
 import System.Posix.Files ( createLink )
-import System.Directory ( createDirectoryIfMissing )
-import System.FilePath.Posix ( (</>), takeDirectory, normalise )
+import System.FilePath.Posix ( (</>), normalise )
 
-import Darcs.Flags ( DarcsFlag( SignAs, Sign, SignSSL, NoLinks,
-                                Verify, VerifySSL, RemoteDarcs ) )
+import Darcs.Flags ( DarcsFlag( SignAs, Sign, SignSSL,
+                                Verify, VerifySSL )
+                   , RemoteDarcs(..) )
 import Darcs.RepoPath ( AbsolutePath, toFilePath )
-import Darcs.Utils ( withCurrentDirectory, breakCommand, getViewer, ortryrunning, )
-import Progress ( withoutProgress, progressList, debugMessage )
+import Darcs.Utils ( breakCommand, getViewer, ortryrunning, )
+import Progress ( withoutProgress, debugMessage )
 
 import ByteStringUtils (gzReadFilePS, linesPS, unlinesPS)
 import qualified Data.ByteString as B (ByteString, empty, null, readFile
@@ -66,16 +66,24 @@ import qualified Data.ByteString.Lazy as BL
 import Darcs.Lock ( withTemp, withOpenTemp, tempdirLoc, removeFileMayNotExist )
 import CommandLine ( parseCmd, addUrlencoded )
 import URL ( copyUrl, copyUrlFirst, waitUrl )
-import Ssh ( getSSH, copySSH, copySSHs, SSHCmd(..) )
 import URL ( Cachable(..) )
 import Exec ( exec, Redirect(..), withoutNonBlock )
-import Darcs.URL ( isFile, isUrl, isSsh )
+import Darcs.Ssh ( getSSH, copySSH, SSHCmd(..) )
+import Darcs.URL ( isFile, isHttpUrl, isSshUrl, splitSshUrl, SshFilePath, sshUhost )
 import Darcs.Utils ( catchall )
 import Printer ( Doc, Printers, putDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), renderPS,
                  simplePrinters,
                  text, empty, packedString, vcat, renderString )
 import Darcs.Email ( formatHeader )
 
+#ifdef HAVE_HTTP
+import Network.Browser ( browse, request, setErrHandler, setOutHandler
+    , setAllowRedirects )
+import Network.HTTP ( RequestMethod(GET), rspCode, rspBody, rspReason
+    , mkRequest )
+import Network.URI ( parseURI, uriScheme )
+#endif
+
 sendmailPath :: IO String
 sendmailPath = do
   l <- filterM doesFileExist $ liftM2 (</>)
@@ -115,8 +123,9 @@ backupBy :: (FilePath -> FilePath -> IO ()) -> FilePath -> IO ()
 backupBy backup f =
            do hasBF <- doesFileExist f
               hasBD <- doesDirectoryExist f
-              when (hasBF || hasBD) $ helper (0::Int)
+              when (hasBF || hasBD) $ helper 0
   where
+  helper :: Int -> IO ()
   helper i = do existsF <- doesFileExist next
                 existsD <- doesDirectoryExist next
                 if (existsF || existsD)
@@ -124,77 +133,59 @@ backupBy backup f =
                    else do putStrLn $ "Backing up " ++ f ++ "(" ++ suffix ++ ")"
                            backup f next
              where next = f ++ suffix
-                   suffix = "-darcs-backup" ++ show i
+                   suffix = ".~" ++ show i ++ "~"
 
--- | @fetchFile fileOrUrl cache@ returns the content of its argument
--- (either a file or an URL). If it has to download an url, then it
--- will use a cache as required by its second argument.
+copyAndReadFile :: (FilePath -> IO a) -> String -> Cachable -> IO a
+copyAndReadFile readfn fou _ | isFile fou = readfn fou
+copyAndReadFile readfn fou cache = withTemp $ \t -> do copyFileOrUrl DefaultRemoteDarcs fou t cache
+                                                       readfn t
+
+-- | @fetchFile fileOrUrl cache@ returns the content of its argument (either a
+-- file or an URL). If it has to download an url, then it will use a cache as
+-- required by its second argument.
+--
+-- We always use default remote darcs, since it is not fatal if the remote
+-- darcs does not exist or is too old -- anything that supports transfer-mode
+-- should do, and if not, we will fall back to SFTP or SCP.
 fetchFilePS :: String -> Cachable -> IO B.ByteString
-fetchFilePS fou _ | isFile fou = B.readFile fou
-fetchFilePS fou cache = withTemp $ \t -> do let opts = [] -- FIXME: no network flags
-                                            copyFileOrUrl opts fou t cache
-                                            B.readFile t
+fetchFilePS = copyAndReadFile B.readFile
 
--- | @fetchFileLazyPS fileOrUrl cache@ lazily reads the content of
--- its argument (either a file or an URL). Warning: this function may
--- constitute a fd leak; make sure to force consumption of file contents
--- to avoid that.
+-- | @fetchFileLazyPS fileOrUrl cache@ lazily reads the content of its argument
+-- (either a file or an URL). Warning: this function may constitute a fd leak;
+-- make sure to force consumption of file contents to avoid that. See
+-- "fetchFilePS" for details.
 fetchFileLazyPS :: String -> Cachable -> IO BL.ByteString
-fetchFileLazyPS fou _ | isFile fou = BL.readFile fou
-fetchFileLazyPS fou cache = withTemp $ \t -> do let opts = [] -- FIXME: no network flags
-                                                copyFileOrUrl opts fou t cache
-                                                BL.readFile t
+#ifdef HAVE_HTTP
+fetchFileLazyPS x c = case parseURI x of
+  Just x' | uriScheme x' == "http:" -> do
+    rsp <- fmap snd . browse $ do
+      setErrHandler . const $ return ()
+      setOutHandler . const $ return ()
+      setAllowRedirects True
+      request $ mkRequest GET x'
+    if rspCode rsp /= (2, 0, 0)
+      then fail $ "fetchFileLazyPS: " ++ rspReason rsp
+      else return $ rspBody rsp
+  _ -> copyAndReadFile BL.readFile x c
+#else
+fetchFileLazyPS x c = copyAndReadFile BL.readFile x c
+#endif
 
 gzFetchFilePS :: String -> Cachable -> IO B.ByteString
-gzFetchFilePS fou _ | isFile fou = gzReadFilePS fou
-gzFetchFilePS fou cache = withTemp $ \t-> do let opts = [] -- FIXME: no network flags
-                                             copyFileOrUrl opts fou t cache
-                                             gzReadFilePS t
+gzFetchFilePS = copyAndReadFile gzReadFilePS
 
-remoteDarcsCmd :: [DarcsFlag] -> String
-remoteDarcsCmd flags = head $ [ c | (RemoteDarcs c) <- flags ] ++ ["darcs"]
-
-copyFileOrUrl :: [DarcsFlag] -> FilePath -> FilePath -> Cachable -> IO ()
-copyFileOrUrl opts fou out _     | isFile fou = copyLocal opts fou out
-copyFileOrUrl _    fou out cache | isUrl  fou = copyRemote fou out cache
-copyFileOrUrl opts fou out _     | isSsh  fou = copySSH (remoteDarcsCmd opts) fou out
+copyFileOrUrl :: RemoteDarcs -> FilePath -> FilePath -> Cachable -> IO ()
+copyFileOrUrl _    fou out _     | isFile fou = copyLocal fou out
+copyFileOrUrl _    fou out cache | isHttpUrl  fou = copyRemote fou out cache
+copyFileOrUrl rd   fou out _     | isSshUrl  fou = copySSH rd (splitSshUrl fou) out
 copyFileOrUrl _    fou _   _     = fail $ "unknown transport protocol: " ++ fou
 
 speculateFileOrUrl :: String -> FilePath -> IO ()
-speculateFileOrUrl fou out | isUrl fou = speculateRemote fou out
+speculateFileOrUrl fou out | isHttpUrl fou = speculateRemote fou out
                            | otherwise = return ()
 
-speculateFilesOrUrls :: FilePath -> [String] -> FilePath -> IO ()
-speculateFilesOrUrls _ _ _ = return () -- FIXME
-
-copyLocal  :: [DarcsFlag] -> String -> FilePath -> IO ()
-copyLocal opts fou out | NoLinks `elem` opts = cloneFile fou out
-                       | otherwise = createLink fou out `catchall` cloneFile fou out
-
-clonePaths :: FilePath -> FilePath -> [FilePath] -> IO ()
-clonePaths source dest = mapM_ (clonePath source dest)
-
-clonePath :: FilePath -> FilePath -> FilePath -> IO ()
-clonePath source dest path
- = do let source' = source </> path
-          dest' = dest </> path
-      fs <- getSymbolicLinkStatus source'
-      if isDirectory fs then do
-          createDirectoryIfMissing True dest'
-       else if isRegularFile fs then do
-          createDirectoryIfMissing True (dest </> takeDirectory path)
-          cloneFile source' dest'
-       else fail ("clonePath: Bad file " ++ source')
-   `catch` fail ("clonePath: Bad file " ++ source </> path)
-
-clonePartialsTree :: FilePath -> FilePath -> [FilePath] -> IO ()
-clonePartialsTree source dest = mapM_ (clonePartialTree source dest)
-
-clonePartialTree :: FilePath -> FilePath -> FilePath -> IO ()
-clonePartialTree source dest "" = cloneTree source dest
-clonePartialTree source dest pref
- = do createDirectoryIfMissing True (dest </> takeDirectory pref)
-      cloneSubTree (source </> pref) (dest </> pref)
+copyLocal  :: String -> FilePath -> IO ()
+copyLocal fou out = createLink fou out `catchall` cloneFile fou out
 
 cloneTree :: FilePath -> FilePath -> IO ()
 cloneTree = cloneTreeExcept []
@@ -221,7 +212,7 @@ cloneSubTree source dest =
             mk_source fp = source </> fp
             mk_dest   fp = dest   </> fp
         zipWithM_ cloneSubTree (map mk_source fps') (map mk_dest fps')
-     else if isRegularFile fs then do
+     else if isRegularFile fs then
         cloneFile source dest
      else fail ("cloneSubTree: Bad source "++ source)
     `catch` (\e -> if isDoesNotExistError e
@@ -262,67 +253,13 @@ copyRemote u v cache =
 copyRemoteNormal :: String -> FilePath -> Cachable -> IO ()
 copyRemoteNormal u v cache = copyUrlFirst u v cache >> waitUrl u
 
-copyFilesOrUrls :: [DarcsFlag]->FilePath->[String]->FilePath->Cachable->IO ()
-copyFilesOrUrls opts dou ns out _ | isFile dou = copyLocals opts dou ns out
-copyFilesOrUrls _ dou ns out c    | isUrl  dou = copyRemotes dou ns out c
-copyFilesOrUrls opts dou ns out _ | isSsh  dou = copySSHs (remoteDarcsCmd opts) dou ns out
-copyFilesOrUrls _ dou _  _   _    = fail $ "unknown transport protocol: "++dou
-
-
-copyLocals :: [DarcsFlag] -> String -> [String] -> FilePath -> IO ()
-copyLocals opts u ns d =
-    doWithPatches (\n -> copyLocal opts (u++"/"++n) (d++"/"++n)) ns
-
-copyRemotes :: String -> [String] -> FilePath -> Cachable -> IO()
-copyRemotes u ns d cache =
-    do maybeget <- maybeURLCmd "GET" u
-       maybemget <- maybeURLCmd "MGET" u
-       case (maybeget, maybemget) of
-         (Nothing, _) -> copyRemotesNormal u ns d cache
-         (Just _, Nothing) -> doWithPatches (\n -> copyRemote (u++"/"++n) (d++"/"++n) cache) ns
-         (Just _, Just mget) -> mgetRemotes mget u ns d
-
-stringToInt :: String -> Int -> Int
-stringToInt num def = case reads num of [(x,"")] -> x
-                                        _ -> def
-
-mgetRemotes :: String -> String -> [String] -> FilePath -> IO()
-mgetRemotes _ _ [] _ = return ()
-mgetRemotes mget u ns d = do
-    mgetmax <- getEnv "DARCS_MGETMAX" `catch` \_ -> return ""
-    let (nsnow, nslater) = splitAt (stringToInt mgetmax 200) ns
-        (cmd, args) = breakCommand mget
-        urls = map (\n -> u++"/"++n) nsnow
-    withCurrentDirectory d $ do
-        r <- exec cmd (args++urls) (Null,Null,AsIs)
-        when (r /= ExitSuccess) $
-            fail $ unlines $
-                ["(" ++ mget ++ ") failed to fetch files.",
-                     "source directory: " ++ d,
-                     "source files:"] ++ (upto 5 nsnow) ++
-                     ["still to go:"] ++ (upto 5 nslater)
-    mgetRemotes mget u nslater d
-    where
-    upto :: Integer -> [String] -> [String]
-    upto _ [] = []
-    upto 0 l = [ "(" ++ (show (length l)) ++ " more)" ]
-    upto n (h : t) = h : (upto (n - 1) t)
-
-copyRemotesNormal :: String -> [String] -> FilePath -> Cachable -> IO()
-copyRemotesNormal u ns d cache =
-    do mapM_ (\n -> copyUrl (u++"/"++n) (d++"/"++n) cache) ns
-       doWithPatches (\n -> waitUrl (u++"/"++n)) ns
-
-doWithPatches :: (String -> IO ()) -> [String] -> IO ()
-doWithPatches f patches = mapM_ (\p -> seq p $ f p) $ progressList "Copying patch" patches
-
 -- | Run a command on a remote location without passing it any input or
 --   reading its output.  Return its ExitCode
-execSSH :: String -> String -> IO ExitCode
+execSSH :: SshFilePath -> String -> IO ExitCode
 execSSH remoteAddr command =
-    do (ssh, ssh_args) <- getSSH SSH remoteAddr
-       debugMessage $ unwords (ssh:ssh_args++[remoteAddr,command])
-       withoutProgress $ do hid <- runProcess ssh (ssh_args++[remoteAddr,command])
+    do (ssh, ssh_args) <- getSSH SSH
+       debugMessage $ unwords (ssh:ssh_args++[sshUhost remoteAddr,command])
+       withoutProgress $ do hid <- runProcess ssh (ssh_args++[sshUhost remoteAddr,command])
                                    Nothing Nothing Nothing Nothing Nothing
                             waitForProcess hid
 
@@ -333,11 +270,13 @@ pipeDoc c args inp = withoutNonBlock $ withoutProgress $
        hSetBinaryMode i True
        hSetBinaryMode o True
        mvare <- newEmptyMVar
-       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        mvaro <- newEmptyMVar
-       forkIO ((Ratified.hGetContents o >>= -- ratify: immediately consumed
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO ((Ratified.hGetContents o >>= -- ratify: immediately consumed
                 hPutStr stdout)
                `finally` putMVar mvaro ())
        hPutDoc i inp
@@ -349,10 +288,10 @@ pipeDoc c args inp = withoutNonBlock $ withoutProgress $
             putStrLn $ "Command not found:\n   "++ show (c:args)
        return rval
 
-pipeDocSSH :: String -> [String] -> Doc -> IO ExitCode
+pipeDocSSH :: SshFilePath -> [String] -> Doc -> IO ExitCode
 pipeDocSSH remoteAddr args input =
-    do (ssh, ssh_args) <- getSSH SSH remoteAddr
-       pipeDoc ssh (ssh_args++ (remoteAddr:args)) input
+    do (ssh, ssh_args) <- getSSH SSH
+       pipeDoc ssh (ssh_args++ (sshUhost remoteAddr:args)) input
 
 sendEmail :: String -> String -> String -> String -> String -> String -> IO ()
 sendEmail f t s cc scmd body =
@@ -397,7 +336,7 @@ sendEmailDoc f "" s cc scmd mbundle body =
   sendEmailDoc f cc s "" scmd mbundle body
 sendEmailDoc f t s cc scmd mbundle body = do
   use_sendmail <- haveSendmail
-  if use_sendmail || scmd /= "" then do
+  if use_sendmail || scmd /= "" then
     withOpenTemp $ \(h,fn) -> do
       generateEmail h f t s cc body
       hClose h
@@ -444,7 +383,7 @@ resendEmail "" _ _ = return ()
 resendEmail t scmd body = do
   use_sendmail <- haveSendmail
   if use_sendmail || scmd /= ""
-   then do
+   then
     withOpenTemp $ \(h,fn) -> do
      hPutStrLn h $ "To: "++ t
      hPutStrLn h $ find_from (linesPS body)
@@ -510,9 +449,11 @@ execPSPipe c args ps = fmap renderPS
 execDocPipe :: String -> [String] -> Doc -> IO Doc
 execDocPipe c args instr = withoutProgress $
     do (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
-       forkIO $ hPutDoc i instr >> hClose i
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO $ hPutDoc i instr >> hClose i
        mvare <- newEmptyMVar
-       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        out <- B.hGetContents o
@@ -528,15 +469,18 @@ execDocPipe c args instr = withoutProgress $
 execPipeIgnoreError :: String -> [String] -> Doc -> IO Doc
 execPipeIgnoreError c args instr = withoutProgress $
     do (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
-       forkIO $ hPutDoc i instr >> hClose i
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO $ hPutDoc i instr >> hClose i
        mvare <- newEmptyMVar
-       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+       _ <- forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        out <- B.hGetContents o
-       waitForProcess pid
+-- Warning:  A do-notation statement discarded a result of type ExitCode.
+       _ <- waitForProcess pid
        takeMVar mvare
-       return $ packedString out
+       return $ if B.null out then empty else packedString out
 
 signString :: [DarcsFlag] -> Doc -> IO Doc
 signString [] d = return d
@@ -646,7 +590,8 @@ viewDoc = viewDocWith simplePrinters
 viewDocWith :: Printers -> Doc -> IO ()
 viewDocWith pr msg = do
   isTerminal <- hIsTerminalDevice stdout
-  if isTerminal && lengthGreaterThan (20 :: Int) (lines $ renderString msg)
+-- Warning:  A do-notation statement discarded a result of type ExitCode.
+  _ <- if isTerminal && lengthGreaterThan (20 :: Int) (lines $ renderString msg)
      then do viewerPlusArgs <- getViewer
              let (viewer:args) = words viewerPlusArgs
              pipeDocToPager viewer args pr msg
@@ -674,11 +619,11 @@ pipeDocToPager c args pr inp = withoutNonBlock $ withoutProgress $ do
     do hPutDocWith pr fh inp
        hClose fh
        bracket (openBinaryFile fn ReadMode) hClose $ \h ->
-         do x <- do pid <- runProcess c args Nothing Nothing (Just h) Nothing Nothing
-                    waitForProcess pid
+         do x <- do waitForProcess  =<< runProcess c args Nothing Nothing (Just h) Nothing Nothing
             when (x == ExitFailure 127) $
                  putStrLn $ "Command not found:\n   "++ show (c:args)
             return x
   where
-    cleanup (f,h) = do try (hClose h) :: IO (Either SomeException ())
+-- Warning:  A do-notation statement discarded a result of type Either SomeException ().
+    cleanup (f,h) = do _ <- try (hClose h) :: IO (Either SomeException ())
                        removeFileMayNotExist f
