@@ -15,18 +15,42 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-module Darcs.Flags ( DarcsFlag( .. ), Compression( .. ), compression,
-                     wantExternalMerge, isInteractive,
-                     maxCount, willIgnoreTimes, willRemoveLogFile, isUnified,
-                     willStoreInMemory, doHappyForwarding, includeBoring,
-                     doAllowCaseOnly, doAllowWindowsReserved, doReverse,
-                     showChangesOnlyToFiles,
-                     defaultFlag,
-                   ) where
-import Data.Maybe( fromMaybe )
+module Darcs.Flags
+    ( DarcsFlag( .. )
+    , Compression( .. )
+    , UseIndex(..)
+    , ScanKnown(..)
+    , RemoteDarcs(..)
+    , compression
+    , remoteDarcs
+    , diffingOpts
+    , wantExternalMerge
+    , wantGuiPause
+    , isInteractive
+    , maxCount
+    , willIgnoreTimes
+    , willRemoveLogFile
+    , isUnified
+    , isNotUnified
+    , doHappyForwarding
+    , includeBoring
+    , doAllowCaseOnly
+    , doAllowWindowsReserved
+    , doReverse
+    , usePacks
+    , showChangesOnlyToFiles
+    , rollbackInWorkingDir
+    , removeFromAmended
+    , defaultFlag
+    ) where
+
+
+import Data.List ( find )
+import Data.Maybe( fromMaybe, isJust )
 import Darcs.Patch.MatchData ( PatchMatch )
 import Darcs.RepoPath ( AbsolutePath, AbsolutePathOrStd )
 
+
 -- | The 'DarcsFlag' type is a list of all flags that can ever be
 -- passed to darcs, or to one of its commands.
 data DarcsFlag = Help | ListOptions | NoTest | Test
@@ -36,14 +60,14 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | Verbose | NormalVerbosity | Quiet
                | Target String | Cc String
                | Output AbsolutePathOrStd | OutputAutoName AbsolutePath
-               | Subject String | InReplyTo String
+               | Subject String | InReplyTo String | Charset String
                | SendmailCmd String | Author String | PatchName String
                | OnePatch String | SeveralPatch String
                | AfterPatch String | UpToPatch String
                | TagName String | LastN Int | MaxCount Int | PatchIndexRange Int Int
                | NumberPatches
                | OneTag String | AfterTag String | UpToTag String
-               | Context AbsolutePath | Count
+               | GenContext | Context AbsolutePath | Count
                | LogFile AbsolutePath | RmLogFile | DontRmLogFile
                | DistName String | All
                | Recursive | NoRecursive | Reorder
@@ -55,8 +79,7 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | Sign | SignAs String | NoSign | SignSSL String
                | HappyForwarding | NoHappyForwarding
                | Verify AbsolutePath | VerifySSL AbsolutePath
-               | SSHControlMaster | NoSSHControlMaster
-               | RemoteDarcs String
+               | RemoteDarcsOpt String
                | EditDescription | NoEditDescription
                | Toks String
                | EditLongComment | NoEditLongComment | PromptLongComment
@@ -75,8 +98,9 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | Pipe | Interactive
                | DiffCmd String
                | ExternalMerge String | Summary | NoSummary
+               | PauseForGui | NoPauseForGui
                | Unified | NonUnified | Reverse | Forward
-               | Partial | Complete | Lazy | Ephemeral
+               | Complete | Lazy
                | FixFilePath AbsolutePath AbsolutePath | DiffFlags String
                | XMLOutput
                | ForceReplace
@@ -92,12 +116,11 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                -- False = defaulted by darcs
                | SetDefault Bool | NoSetDefault Bool
 
-               | FancyMoveAdd | NoFancyMoveAdd
                | Disable | SetScriptsExecutable | DontSetScriptsExecutable | Bisect
-               | UseHashedInventory | UseOldFashionedInventory
-               | UseFormat2
-               | PristinePlain | PristineNone | NoUpdateWorking
-               | Sibling AbsolutePath | Relink | RelinkPristine | NoLinks
+               | UseHashedInventory
+               | UseFormat2 | UseNoWorkingDir | UseWorkingDir
+               | NoUpdateWorking
+               | Sibling AbsolutePath | Relink
                | OptimizePristine | OptimizeHTTP
                | UpgradeFormat
                | Files | NoFiles | Directories | NoDirectories
@@ -107,22 +130,79 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | UMask String
                | StoreInMemory | ApplyOnDisk
                | NoHTTPPipelining
+               | Packs | NoPacks
                | NoCache
                | AllowUnrelatedRepos
                | Check | Repair | JustThisRepo
                | NullFlag
+               | RecordRollback | NoRecordRollback
+               | NoAmendUnrecord | AmendUnrecord
                  deriving ( Eq, Show )
 
-data Compression = NoCompression | GzipCompression
-compression :: [DarcsFlag] -> Compression
-compression f | NoCompress `elem` f = NoCompression
-              | otherwise = GzipCompression
+-- ADTs for selecting specific behaviour... FIXME These should be eventually
+-- moved out from this module, closer to where they are actually used
+
+data Compression = NoCompression
+                 | GzipCompression
+
+data UseIndex = UseIndex
+              | IgnoreIndex
+
+data ScanKnown = ScanKnown -- ^Just files already known to darcs
+               | ScanAll -- ^All files, i.e. look for new ones
+               | ScanBoring -- ^All files, even boring ones
+
+data RemoteDarcs = RemoteDarcs String
+                 | DefaultRemoteDarcs
+
+
+compression :: [DarcsFlag]
+            -> Compression
+compression f
+    | NoCompress `elem` f = NoCompression
+    | otherwise           = GzipCompression
+
+
+remoteDarcs :: [DarcsFlag]
+            -> RemoteDarcs
+remoteDarcs f
+    | (x:_) <- [ c | RemoteDarcsOpt c <- f ] = RemoteDarcs x
+    | otherwise = DefaultRemoteDarcs
+
+
+diffingOpts :: [DarcsFlag]
+            -> (UseIndex, ScanKnown)
+diffingOpts opts = (index, scan)
+  where
+    index = if willIgnoreTimes opts
+            then IgnoreIndex
+            else UseIndex
+    scan =
+        if LookForAdds `elem` opts
+        then
+          if Boring `elem` opts
+          then ScanBoring
+          else ScanAll
+        else ScanKnown
+
 
 wantExternalMerge :: [DarcsFlag] -> Maybe String
 wantExternalMerge [] = Nothing
 wantExternalMerge (ExternalMerge c:_) = Just c
 wantExternalMerge (_:fs) = wantExternalMerge fs
 
+
+wantGuiPause :: [DarcsFlag] -> Bool
+wantGuiPause fs = (hasDiffCmd || hasExternalMerge) && hasPause
+  where
+    hasDiffCmd = any isDiffCmd fs
+    hasExternalMerge = isJust $ wantExternalMerge fs
+    isDiffCmd (DiffCmd _) = True
+    isDiffCmd _ = False
+    hasPause = maybe True (==PauseForGui) $ find isPauseFlag $ reverse fs
+    isPauseFlag f = (f==PauseForGui) || (f==NoPauseForGui)
+
+
 isInteractive :: [DarcsFlag] -> Bool
 isInteractive = isInteractive_ True
     where
@@ -140,6 +220,10 @@ maxCount [] = Nothing
 -- | @lastWord [(flag, value)] default opts@ scans @opts@ for a flag
 -- in the list and returns the value of the first match, or @default@
 -- if none is found.
+--
+-- We call this the \"last\" word because we assume that flags are
+-- *prepended* in the order they arrive, so what is first internally
+-- is last from the user's point of view.
 lastWord :: [(DarcsFlag,a)] -> a -> [DarcsFlag] -> a
 lastWord known_flags = foldr . flip $ \ def -> fromMaybe def . flip lookup known_flags
 
@@ -155,8 +239,8 @@ willRemoveLogFile = getBoolFlag RmLogFile DontRmLogFile
 isUnified :: [DarcsFlag] -> Bool
 isUnified = getBoolFlag Unified NonUnified
 
-willStoreInMemory :: [DarcsFlag] -> Bool
-willStoreInMemory = getBoolFlag Unified NonUnified
+isNotUnified :: [DarcsFlag] -> Bool
+isNotUnified = getBoolFlag NonUnified Unified
 
 doHappyForwarding :: [DarcsFlag] -> Bool
 doHappyForwarding = getBoolFlag HappyForwarding NoHappyForwarding
@@ -174,6 +258,9 @@ doAllowWindowsReserved = getBoolFlag AllowWindowsReserved DontAllowWindowsReserv
 doReverse :: [DarcsFlag] -> Bool
 doReverse = getBoolFlag Reverse Forward
 
+usePacks :: [DarcsFlag] -> Bool
+usePacks = getBoolFlag Packs NoPacks
+
 showChangesOnlyToFiles :: [DarcsFlag] -> Bool
 showChangesOnlyToFiles = getBoolFlag OnlyChangesToFiles ChangesToAllFiles
 
@@ -184,3 +271,9 @@ defaultFlag :: [DarcsFlag] -- ^ distractors
             -> [DarcsFlag] -- ^ updated flags
 defaultFlag alts def flags =
  if any (`elem` flags) alts then flags else def : flags
+
+rollbackInWorkingDir :: [DarcsFlag] -> Bool
+rollbackInWorkingDir = getBoolFlag NoRecordRollback RecordRollback
+
+removeFromAmended :: [DarcsFlag] -> Bool
+removeFromAmended = getBoolFlag AmendUnrecord NoAmendUnrecord
