@@ -25,11 +25,11 @@ module Darcs.Patch.FileName ( FileName( ),
                               movedirfilename,
                               encodeWhite, decodeWhite,
                               (///),
-                              breakup
+                              breakup, isParentOrEqOf
                             ) where
 
-import System.IO
 import Data.Char ( isSpace, chr, ord )
+import Data.List ( stripPrefix )
 import ByteStringUtils ( packStringToUTF8, unpackPSFromUTF8 )
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 import qualified Data.ByteString       as B  (ByteString)
@@ -143,13 +143,19 @@ breakLast c l = case breakFirst c (reverse l) of
 (FN "")///b = normPath b
 a///b = normPath $ fp2fn $ fn2fp a ++ "/" ++ fn2fp b
 
+isParentOrEqOf :: FileName -> FileName -> Bool
+isParentOrEqOf fn1 fn2 = case stripPrefix (fn2fp fn1) (fn2fp fn2) of
+    Just ('/' : _) -> True
+    Just [] -> True
+    _ -> False
+
 movedirfilename :: FileName -> FileName -> FileName -> FileName
 movedirfilename old new name =
-    if name' == old' then new
-                     else if length name' > length old' &&
-                             take (length old'+1) name' == old'++"/"
-                          then fp2fn ("./"++new'++drop (length old') name')
-                          else name
-    where old' = fn2fp $ normPath old
-          new' = fn2fp $ normPath new
-          name' = fn2fp $ normPath name
+    if name' == old'
+        then new
+        else case stripPrefix old' name' of
+            Just rest@('/':_) -> fp2fn $ "./" ++ new' ++ rest
+            _ -> name
+        where old' = fn2fp $ normPath old
+              new' = fn2fp $ normPath new
+              name' = fn2fp $ normPath name
