@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-module Darcs.Patch.MatchData ( PatchMatch(..), patch_match,
+module Darcs.Patch.MatchData ( PatchMatch(..), patchMatch,
                       ) where
 
 data PatchMatch = PatternMatch String
@@ -24,5 +24,5 @@ data PatchMatch = PatternMatch String
 instance Show PatchMatch where
     show (PatternMatch m) = "pattern " ++ show m
 
-patch_match :: String -> PatchMatch
-patch_match s = PatternMatch s
+patchMatch :: String -> PatchMatch
+patchMatch s = PatternMatch s
