@@ -113,34 +113,34 @@ dateTime tz =
 cvsDateTime :: Int -> CharParser a CalendarTime
 cvsDateTime tz =
                 do y <- year
-                   char '/'
+                   _ <- char '/'
                    mon <- monthNum
-                   char '/'
+                   _ <- char '/'
                    d <- day
-                   mySpaces
+                   _ <- mySpaces
                    h <- hour
-                   char ':'
+                   _ <- char ':'
                    m <- minute
-                   char ':'
+                   _ <- char ':'
                    s <- second
                    z <- option tz $ mySpaces >> zone
                    return (CalendarTime y mon d h m s 0 Monday 0 "" z False)
 
 oldDateTime   :: CharParser a CalendarTime
 oldDateTime    =   do wd <- dayName
-                      mySpaces
+                      _ <- mySpaces
                       mon <- monthName
-                      mySpaces
+                      _ <- mySpaces
                       d <- day
-                      mySpaces
+                      _ <- mySpaces
                       h <- hour
-                      char ':'
+                      _ <- char ':'
                       m <- minute
-                      char ':'
+                      _ <- char ':'
                       s <- second
-                      mySpaces
+                      _ <- mySpaces
                       z <- zone
-                      mySpaces
+                      _ <- mySpaces
                       y <- year
                       return (CalendarTime y mon d h m s 0 wd 0 "" z False)
 
@@ -181,7 +181,7 @@ iso8601Date =
     week_date = --yyyy-Www-dd
       try $ do yfn <- year_
                optional dash
-               char 'W'
+               _ <- char 'W'
                -- offset human 'week 1' -> computer 'week 0'
                w'  <- (\x -> x-1) `liftM` twoDigits
                wd  <- option 1 $ do { optional dash; nDigits 1 }
@@ -233,7 +233,7 @@ iso8601Time = try $
                         | digsExp > picoExp = read $ take picoExp digs
                         | otherwise = 10 ^ (picoExp - digsExp) * (read digs)
                return $ \c -> c { ctPicosec = frac }
-    zulu   = do { char 'Z'; return (\c -> c { ctTZ = 0 }) }
+    zulu   = do { _ <- char 'Z'; return (\c -> c { ctTZ = 0 }) }
     offset = do sign <- choice [ do { char '+' >> return   1  }
                                , do { char '-' >> return (-1) } ]
                 h <- twoDigits
@@ -323,8 +323,8 @@ second           = twoDigits
 
 zone            :: CharParser a Int
 zone             = choice
-                       [ do { char '+'; h <- hour; m <- minute; return (((h*60)+m)*60) }
-                       , do { char '-'; h <- hour; m <- minute; return (-((h*60)+m)*60) }
+                       [ do { _ <- char '+'; h <- hour; m <- minute; return (((h*60)+m)*60) }
+                       , do { _ <- char '-'; h <- hour; m <- minute; return (-((h*60)+m)*60) }
                        , mkZone "UTC"  0
                        , mkZone "UT"  0
                        , mkZone "GMT" 0
@@ -339,12 +339,12 @@ zone             = choice
                        , mkZone "CEST" 2
                        , mkZone "EEST" 3
                          -- if we don't understand it, just give a GMT answer...
-                       , do { manyTill (oneOf $ ['a'..'z']++['A'..'Z']++[' '])
+                       , do { _ <- manyTill (oneOf $ ['a'..'z']++['A'..'Z']++[' '])
                                        (lookAhead space_digit);
                               return 0 }
                        ]
      where mkZone n o  = try $ do { caseString n; return (o*60*60) }
-           space_digit = try $ do { char ' '; oneOf ['0'..'9'] }
+           space_digit = try $ do { _ <- char ' '; oneOf ['0'..'9'] }
 
 nullCalendar :: CalendarTime
 nullCalendar = CalendarTime 0 January 0 0 0 0 1 Sunday 0 "" 0 False
