@@ -1,8 +1,8 @@
-{-# OPTIONS_GHC -cpp #-}
 {-# LANGUAGE CPP, PatternGuards #-}
 
-module Darcs.Repository.Repair ( replayRepository, checkIndex
-                               , RepositoryConsistency(..) )
+module Darcs.Repository.Repair ( replayRepository, checkIndex,
+                                 replayRepositoryInTemp,
+                                 RepositoryConsistency(..) )
        where
 
 import Control.Monad ( when, unless )
@@ -11,23 +11,26 @@ import Control.Applicative( (<$>) )
 import Control.Exception ( finally )
 import Data.Maybe ( catMaybes )
 import Data.List ( sort, (\\) )
-import System.Directory ( createDirectoryIfMissing )
-
+import System.Directory ( createDirectoryIfMissing, getCurrentDirectory,
+                          setCurrentDirectory )
 import Darcs.Lock( rmRecursive )
-import Darcs.Hopefully ( PatchInfoAnd, info, winfo, WPatchInfo, unWPatchInfo, compareWPatchInfo )
+import Darcs.RepoPath( AbsolutePath, ioAbsolute, toFilePath )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, winfo, WPatchInfo, unWPatchInfo, compareWPatchInfo )
 
+import Darcs.Witnesses.Eq ( EqCheck(..) )
 import Darcs.Witnesses.Ordered ( FL(..), RL(..), lengthFL, reverseFL,
-                                 mapRL, nullFL, (:||:)(..), EqCheck(..) )
+                                 mapRL, nullFL, (:||:)(..) )
 import Darcs.Witnesses.Sealed ( Sealed2(..), Sealed(..), unFreeLeft )
-import Darcs.Patch.Patchy ( applyAndTryToFix )
-import Darcs.Hopefully( hopefully )
+import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Repair ( Repair(applyAndTryToFix) )
+import Darcs.Patch.PatchInfoAnd( hopefully )
 import Darcs.Patch.Info ( humanFriendly )
 import Darcs.Patch.Set ( PatchSet(..), newset2FL, newset2RL )
 #ifdef GADT_WITNESSES
 import Darcs.Patch.Set ( Origin )
 #endif
-import Darcs.Patch ( RepoPatch, isInconsistent )
-import Darcs.Patch.Core ( patchcontents )
+import Darcs.Patch ( RepoPatch, PrimOf, isInconsistent )
+import Darcs.Patch.Named ( patchcontents )
 
 import Darcs.Repository.Format ( identifyRepoFormat,
                                  RepoProperty ( HashedInventory ), formatHas )
@@ -42,6 +45,7 @@ import Darcs.Repository ( Repository, readRepo, makePatchLazy
 import Progress ( debugMessage, beginTedious, endTedious, tediousSize, finishedOneIO )
 import Darcs.Utils ( catchall )
 import Darcs.Global ( darcsdir )
+import Darcs.Lock( withTempDir )
 import Printer ( Doc, putDocLn, text )
 import Darcs.Arguments ( DarcsFlag( Verbose, Quiet ) )
 
@@ -70,7 +74,9 @@ replaceInFL (o:>:orig) ch@(Sealed2 (o':||:c):ch_rest)
     | IsEq <- winfo o `compareWPatchInfo` o' = c:>:replaceInFL orig ch_rest
     | otherwise = o:>:replaceInFL orig ch
 
-applyAndFix :: forall p C(r u t x y). RepoPatch p => Repository p C(r u t) -> FL (PatchInfoAnd p) C(Origin r) -> TreeIO (FL (PatchInfoAnd p) C(Origin r), Bool)
+applyAndFix :: forall p C(r u t). (RepoPatch p, ApplyState p ~ Tree)
+            => Repository p C(r u t) -> FL (PatchInfoAnd p) C(Origin r)
+            -> TreeIO (FL (PatchInfoAnd p) C(Origin r), Bool)
 applyAndFix _ NilFL = return (NilFL, True)
 applyAndFix r psin =
     do liftIO $ beginTedious k
@@ -92,16 +98,17 @@ applyAndFix r psin =
             (ps', restok) <- aaf ps
             case mp' of
               Nothing -> return (ps', restok)
-              Just (e,pp) -> do liftIO $ putStrLn e
-                                p' <- liftIO $ makePatchLazy r pp
-                                return (Sealed2 (winfp :||: p'):ps', False)
+              Just (e,pp) -> liftIO $ do putStrLn e
+                                         p' <- makePatchLazy r pp
+                                         return (Sealed2 (winfp :||: p'):ps', False)
 
 data RepositoryConsistency p C(x) =
     RepositoryConsistent
   | BrokenPristine (Tree IO)
   | BrokenPatches (Tree IO) (PatchSet p C(Origin x))
 
-checkUniqueness :: RepoPatch p => (Doc -> IO ()) -> (Doc -> IO ()) -> Repository p C(r u t) -> IO ()
+checkUniqueness :: (RepoPatch p, ApplyState p ~ Tree)
+                => (Doc -> IO ()) -> (Doc -> IO ()) -> Repository p C(r u t) -> IO ()
 checkUniqueness putVerbose putInfo repository =
     do putVerbose $ text "Checking that patch names are unique..."
        r <- readRepo repository
@@ -117,13 +124,15 @@ hasDuplicate li = hd $ sort li
           hd [] = Nothing
           hd (x1:x2:xs) | x1 == x2 = Just x1
                         | otherwise = hd (x2:xs)
-replayRepository' :: (RepoPatch p)
-                  => Repository p C(r u t) -> [DarcsFlag] -> IO (RepositoryConsistency p C(r))
-replayRepository' repo opts = do
-  let putVerbose s = when (Verbose `elem` opts) $ putDocLn s
+replayRepository' ::
+    forall p C(r u t) . (RepoPatch p, ApplyState p ~ Tree)
+               => AbsolutePath -> Repository p C(r u t) -> [DarcsFlag] -> IO (RepositoryConsistency p C(r))
+replayRepository' whereToReplay' repo opts = do
+  let whereToReplay = toFilePath whereToReplay'
+      putVerbose s = when (Verbose `elem` opts) $ putDocLn s
       putInfo s = when (not $ Quiet `elem` opts) $ putDocLn s
   checkUniqueness putVerbose putInfo repo
-  createDirectoryIfMissing False $ darcsdir ++ "/pristine.hashed"
+  createDirectoryIfMissing False whereToReplay
   putVerbose $ text "Reading recorded state..."
   pris <- readRecorded repo `catch` \_ -> return emptyTree
   putVerbose $ text "Applying patches..."
@@ -131,13 +140,14 @@ replayRepository' repo opts = do
   debugMessage "Fixing any broken patches..."
   let psin = newset2FL patches
       repair = applyAndFix repo psin
-  ((ps, patches_ok), newpris) <- hashedTreeIO repair emptyTree "_darcs/pristine.hashed"
+
+  ((ps, patches_ok), newpris) <- hashedTreeIO repair emptyTree whereToReplay
   debugMessage "Done fixing broken patches..."
   let newpatches = PatchSet (reverseFL ps) NilRL
 
   debugMessage "Checking pristine against slurpy"
   ftf <- filetypeFunction
-  is_same <- do Sealed diff <- unFreeLeft `fmap` treeDiff ftf pris newpris
+  is_same <- do Sealed diff <- unFreeLeft `fmap` treeDiff ftf pris newpris :: IO (Sealed (FL (PrimOf p) C(r)))
                 return $ nullFL diff
               `catchall` return False
   -- TODO is the latter condition needed? Does a broken patch imply pristine
@@ -151,22 +161,33 @@ replayRepository' repo opts = do
 cleanupRepositoryReplay :: Repository p C(r u t) -> IO ()
 cleanupRepositoryReplay r = do
   let c = extractCache r
-  rf_or_e <- identifyRepoFormat "."
-  rf <- case rf_or_e of Left e -> fail e
-                        Right x -> return x
+  rf <- identifyRepoFormat "."
   unless (formatHas HashedInventory rf) $
          rmRecursive $ darcsdir ++ "/pristine.hashed"
   when (formatHas HashedInventory rf) $ do
        current <- readHashedPristineRoot r
        cleanHashdir c HashedPristineDir $ catMaybes [current]
 
-replayRepository :: (RepoPatch p) => Repository p C(r u t) -> [DarcsFlag] -> (RepositoryConsistency p C(r) -> IO a) -> IO a
-replayRepository r opt f = run `finally` cleanupRepositoryReplay r
+replayRepositoryInTemp :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> [DarcsFlag]
+                          -> IO (RepositoryConsistency p C(r))
+replayRepositoryInTemp r opt = do
+  repodir <- getCurrentDirectory
+  withTempDir "darcs-check" $ \tmpDir -> do
+    setCurrentDirectory repodir
+    replayRepository' tmpDir r opt
+
+replayRepository :: (RepoPatch p, ApplyState p ~ Tree)
+                 => Repository p C(r u t) -> [DarcsFlag]
+                 -> (RepositoryConsistency p C(r) -> IO a) -> IO a
+replayRepository r opt f = do
+  run `finally` cleanupRepositoryReplay r
     where run = do
-            st <- replayRepository' r opt
+            createDirectoryIfMissing False "_darcs/pristine.hashed"
+            hashedPristine <- ioAbsolute "_darcs/pristine.hashed"
+            st <- replayRepository' hashedPristine r opt
             f st
 
-checkIndex :: (RepoPatch p) => Repository p C(r u t) -> Bool -> IO Bool
+checkIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> Bool -> IO Bool
 checkIndex repo quiet = do
   index <- updateIndex =<< readIndex repo
   pristine <- expand =<< readRecordedAndPending repo
