@@ -1,5 +1,5 @@
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE CPP #-}
-{-# OPTIONS_GHC -cpp -fglasgow-exts -fno-warn-orphans #-}
 
 -- Copyright (C) 2009 Petr Rockai
 --
@@ -30,7 +30,7 @@ module Darcs.Repository.State
     -- * Trees.
     , readRecorded, readUnrecorded, readRecordedAndPending, readWorking
     -- * Index.
-    , readIndex, invalidateIndex ) where
+    , readIndex, invalidateIndex, UseIndex(..), ScanKnown(..) ) where
 
 import Prelude hiding ( filter )
 import Control.Monad( when )
@@ -44,18 +44,19 @@ import System.FilePath ( (</>) )
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as BSC
 
-import Darcs.Patch ( RepoPatch, Prim, invert, applyToTree, applyToFilepaths
-                   , sortCoalesceFL )
-import Darcs.Patch.TouchesFiles ( chooseTouching )
-import Darcs.Witnesses.Ordered ( FL(..), (+>+) )
-import Darcs.Witnesses.Ordered ( unsafeCoerceP, EqCheck(IsEq) )
-import Darcs.Witnesses.Sealed ( Sealed(Sealed), seal, unFreeLeft )
+import Darcs.Patch ( RepoPatch, PrimOf, sortCoalesceFL, fromPrim, effect )
+import Darcs.Patch.Apply ( ApplyState, applyToTree, effectOnFilePaths )
+import Darcs.Witnesses.Ordered ( FL(..), (+>+), mapFL_FL )
+import Darcs.Witnesses.Eq ( EqCheck(IsEq, NotEq) )
+import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Witnesses.Sealed ( Sealed(Sealed), seal, unFreeLeft, mapSeal )
 import Darcs.Diff ( treeDiff )
-import Darcs.Flags ( DarcsFlag( LookForAdds ), willIgnoreTimes )
+import Darcs.Flags ( UseIndex(..), ScanKnown(..) )
 import Darcs.Global ( darcsdir )
 import Darcs.Utils ( filterPaths )
 
-import Darcs.Repository.InternalTypes ( Repository )
+import Darcs.Repository.InternalTypes ( Repository(..) )
+import Darcs.Repository.Format(formatHas, RepoProperty(NoWorkingDir))
 import qualified Darcs.Repository.LowLevel as LowLevel
 import Darcs.Repository.Prefs ( filetypeFunction, boringRegexps )
 
@@ -74,23 +75,35 @@ import qualified Storage.Hashed.Tree as Tree
 
 newtype TreeFilter m = TreeFilter { applyTreeFilter :: forall tr . FilterTree tr m => tr m -> tr m }
 
+-- TODO: We wrap the pending patch inside RepoPatch here, to avoid the
+-- requirement to propagate an (ApplyState (PrimOf p) ~ ApplyState p)
+-- constraint everywhere. When we have GHC 7.2 as a minimum requirement, we can
+-- lift this constraint into RepoPatch superclass context and remove this hack.
+readPendingLL :: (RepoPatch p, ApplyState p ~ Tree)
+              => Repository p C(r u t) -> IO (Sealed ((FL p) C(t)))
+readPendingLL repo = mapSeal (mapFL_FL fromPrim) `fmap` LowLevel.readPending repo
+
 -- | From a repository and a list of SubPath's, construct a filter that can be
 -- used on a Tree (recorded or unrecorded state) of this repository. This
 -- constructed filter will take pending into account, so the subpaths will be
--- translated correctly relative to pending move patches. As an exception for
--- convenience, if the subpath list is empty, the filter constructed is an
--- identity.
-restrictSubpaths :: (RepoPatch p) => Repository p C(r u t) -> [SubPath]
+-- translated correctly relative to pending move patches.
+restrictSubpaths :: forall p m C(r u t). (RepoPatch p, ApplyState p ~ Tree)
+                 => Repository p C(r u t) -> [SubPath]
                  -> IO (TreeFilter m)
 restrictSubpaths repo subpaths = do
-  Sealed pending <- LowLevel.readPending repo
+  Sealed pending <- readPendingLL repo
   let paths = map (fn2fp . sp2fn) subpaths
-      paths' = paths `union` applyToFilepaths pending paths
+      paths' = paths `union` (effectOnFilePaths pending paths)
       anchored = map floatPath paths'
-      restrictPaths :: FilterTree t m => t m -> t m
-      restrictPaths = if null subpaths then id else filter (filterPaths anchored)
+      restrictPaths :: FilterTree tree m => tree m -> tree m
+      restrictPaths = filter (filterPaths anchored)
   return (TreeFilter restrictPaths)
 
+-- |Is the given path in (or equal to) the _darcs metadata directory?
+inDarcsDir :: AnchoredPath -> Bool
+inDarcsDir (AnchoredPath (Name x:_)) | x == BSC.pack darcsdir = True
+inDarcsDir _ = False
+
 -- | Construct a Tree filter that removes any boring files the Tree might have
 -- contained. Additionally, you should (in most cases) pass an (expanded) Tree
 -- that corresponds to the recorded content of the repository. This is
@@ -105,7 +118,7 @@ restrictSubpaths repo subpaths = do
 restrictBoring :: forall m . Tree m -> IO (TreeFilter m)
 restrictBoring guide = do
   boring <- boringRegexps
-  let boring' (AnchoredPath (Name x:_)) | x == BSC.pack darcsdir = False
+  let boring' p | inDarcsDir p = False
       boring' p = not $ any (\rx -> isJust $ matchRegex rx p') boring
           where p' = anchorPath "" p
       restrictTree :: FilterTree t m => t m -> t m
@@ -114,11 +127,16 @@ restrictBoring guide = do
                                         _ -> True
   return (TreeFilter restrictTree)
 
--- | For a repository and a list of paths (when empty, take everything) compute
--- a (forward) list of prims (i.e. a patch) going from the recorded state of
--- the repository (pristine) to the unrecorded state of the repository (the
--- working copy + pending). When a non-empty list of paths is given, at least
--- the files that live under any of these paths in either recorded or
+-- | Construct a Tree filter that removes any darcs metadata files the
+-- Tree might have contained.
+restrictDarcsdir :: forall m . TreeFilter m
+restrictDarcsdir = TreeFilter $ filter $ \p _ -> not (inDarcsDir p)
+
+-- | For a repository and an optional list of paths (when Nothing, take
+-- everything) compute a (forward) list of prims (i.e. a patch) going from the
+-- recorded state of the repository (pristine) to the unrecorded state of the
+-- repository (the working copy + pending). When a list of paths is given, at
+-- least the files that live under any of these paths in either recorded or
 -- unrecorded will be included in the resulting patch. NB. More patches may be
 -- included in this list, eg. the full contents of the pending patch. This is
 -- usually not a problem, since selectChanges will properly filter the results
@@ -134,31 +152,51 @@ restrictBoring guide = do
 -- is very inefficient, although in extremely rare cases, the index could go
 -- out of sync (file is modified, index is updated and file is modified again
 -- within a single second).
-unrecordedChanges :: FORALL(p r u t) (RepoPatch p)
-                  => [DarcsFlag] -> Repository p C(r u t)
-                  -> [SubPath] -> IO (FL Prim C(t u))
-unrecordedChanges opts repo paths = do
-  (all_current, Sealed (pending :: FL Prim C(t x))) <- readPending repo
+unrecordedChanges :: forall p C(r u t) . (RepoPatch p, ApplyState p ~ Tree)
+                  => (UseIndex, ScanKnown) -> Repository p C(r u t)
+                  -> Maybe [SubPath] -> IO (FL (PrimOf p) C(t u))
+unrecordedChanges _ r@(Repo _ _ rf _) _
+  | (formatHas NoWorkingDir rf) = do
+    IsEq <- return $ workDirLessRepoWitness r
+    return NilFL
+unrecordedChanges (useidx, scan) repo mbpaths = do
+  (all_current, Sealed (pending :: FL p C(t x))) <- readPending repo
 
-  relevant <- restrictSubpaths repo paths
+  relevant <- maybe (return $ TreeFilter id) (restrictSubpaths repo) mbpaths
   let getIndex = I.updateIndex =<< (applyTreeFilter relevant <$> readIndex repo)
       current = applyTreeFilter relevant all_current
 
-  working <- case (LookForAdds `elem` opts, willIgnoreTimes opts) of
-               (False, False) -> getIndex
-               (False, True) -> do
-                 guide <- expand current
-                 applyTreeFilter relevant <$> restrict guide <$> readPlainTree "."
-               (True, ignoretimes) -> do
-                 index <- getIndex
-                 nonboring <- restrictBoring index
-                 plain <- applyTreeFilter relevant <$> applyTreeFilter nonboring <$> readPlainTree "."
-                 return $ if ignoretimes then plain else plain `overlay` index
+  index <- getIndex
+  working <- applyTreeFilter restrictDarcsdir <$> case scan of
+    ScanKnown -> case useidx of
+      UseIndex -> getIndex
+      IgnoreIndex -> do
+        guide <- expand current
+        applyTreeFilter relevant <$> restrict guide <$> readPlainTree "."
+    ScanAll -> do
+      nonboring <- restrictBoring index
+      plain <- applyTreeFilter relevant <$> applyTreeFilter nonboring <$> readPlainTree "."
+      return $ case useidx of
+        UseIndex -> plain `overlay` index
+        IgnoreIndex -> plain
+    ScanBoring -> do
+      plain <- applyTreeFilter relevant <$> readPlainTree "."
+      return $ case useidx of
+        UseIndex -> plain `overlay` index
+        IgnoreIndex -> plain
 
   ft <- filetypeFunction
-  Sealed (diff :: FL Prim C(x y)) <- (unFreeLeft `fmap` treeDiff ft current working) :: IO (Sealed (FL Prim C(x)))
+  Sealed (diff :: FL (PrimOf p) C(x y)) <- (unFreeLeft `fmap` treeDiff ft current working) :: IO (Sealed (FL (PrimOf p) C(x)))
   IsEq <- return (unsafeCoerceP IsEq) :: IO (EqCheck C(y u))
-  return $ sortCoalesceFL (pending +>+ diff)
+  return $ sortCoalesceFL (effect pending +>+ diff)
+
+-- | Witnesses the fact that in the absence of a working directory, we
+-- pretend that the working dir updates magically to the tentative state.
+workDirLessRepoWitness :: Repository p C(r u t)
+                          -> (EqCheck C(u t))
+workDirLessRepoWitness (Repo _ _ rf _) | formatHas NoWorkingDir rf =
+  unsafeCoerceP IsEq
+                                       | otherwise = NotEq
 
 -- | Obtains a Tree corresponding to the "recorded" state of the repository:
 -- this is the same as the pristine cache, which is the same as the result of
@@ -168,7 +206,7 @@ unrecordedChanges opts repo paths = do
 -- no-pristine case, as that requires replaying patches. Cf. 'readDarcsHashed'
 -- and 'readPlainTree' in hashed-storage that are used to do the actual 'Tree'
 -- construction.
-readRecorded :: (RepoPatch p) => Repository p C(r u t) -> IO (Tree IO)
+readRecorded :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> IO (Tree IO)
 readRecorded _repo = do
   let h_inventory = darcsdir </> "hashed_inventory"
   hashed <- doesFileExist h_inventory
@@ -190,15 +228,16 @@ readRecorded _repo = do
                (_, _) -> fail "No pristine tree is available!"
 
 -- | Obtains a Tree corresponding to the "unrecorded" state of the repository:
--- the working tree plus the "pending" patch. The optional list of paths (it is
--- ignored if empty) allows to restrict the query to a subtree.
+-- the working tree plus the "pending" patch. The optional list of paths allows
+-- to restrict the query to a subtree.
 --
 -- Limiting the query may be more efficient, since hashes on the uninteresting
 -- parts of the index do not need to go through an up-to-date check (which
 -- involves a relatively expensive lstat(2) per file.
-readUnrecorded :: (RepoPatch p) => Repository p C(r u t) -> [SubPath] -> IO (Tree IO)
-readUnrecorded repo paths = do
-  relevant <- restrictSubpaths repo paths
+readUnrecorded :: (RepoPatch p, ApplyState p ~ Tree)
+               => Repository p C(r u t) -> Maybe [SubPath] -> IO (Tree IO)
+readUnrecorded repo mbpaths = do
+  relevant <- maybe (return $ TreeFilter id) (restrictSubpaths repo) mbpaths
   readIndex repo >>= I.updateIndex . applyTreeFilter relevant
 
 -- | Obtains a Tree corresponding to the working copy of the
@@ -209,15 +248,14 @@ readWorking :: IO (Tree IO)
 readWorking = expand =<< (nodarcs `fmap` readPlainTree ".")
   where nodarcs = Tree.filter (\(AnchoredPath (Name x:_)) _ -> x /= BSC.pack "_darcs")
 
-readRecordedAndPending :: (RepoPatch p) => Repository p C(r u t) -> IO (Tree IO)
-readRecordedAndPending repo = do
-  pristine <- readRecorded repo
-  Sealed pending <- snd `fmap` readPending repo
-  applyToTree pending pristine
+readRecordedAndPending :: (RepoPatch p, ApplyState p ~ Tree)
+                       => Repository p C(r u t) -> IO (Tree IO)
+readRecordedAndPending repo = fst `fmap` readPending repo
 
-readPending :: (RepoPatch p) => Repository p C(r u t) -> IO (Tree IO, Sealed (FL Prim C(t)))
+readPending :: (RepoPatch p, ApplyState p ~ Tree)
+            => Repository p C(r u t) -> IO (Tree IO, Sealed (FL p C(t)))
 readPending repo =
-  do Sealed pending <- LowLevel.readPending repo
+  do Sealed pending <- readPendingLL repo
      pristine <- readRecorded repo
      catch ((\t -> (t, seal pending)) `fmap` applyToTree pending pristine) $ \ err -> do
        putStrLn $ "Yikes, pending has conflicts! " ++ show err
@@ -235,7 +273,7 @@ invalidateIndex :: t -> IO ()
 invalidateIndex _ = do
   BS.writeFile "_darcs/index_invalid" BS.empty
 
-readIndex :: (RepoPatch p) => Repository p C(r u t) -> IO I.Index
+readIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> IO I.Index
 readIndex repo = do
   invalid <- doesFileExist "_darcs/index_invalid"
   exist <- doesFileExist "_darcs/index"
