@@ -15,53 +15,31 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP #-}
--- , MagicHash, TypeOperators, GADTs #-}
 
 #include "gadts.h"
 
-module Darcs.Witnesses.Ordered ( EqCheck(..), isEq, (:>)(..), (:<)(..), (:\/:)(..), (:/\:)(..), (:||:)(..),
-                             FL(..), RL(..),Proof(..),
+module Darcs.Witnesses.Ordered ( (:>)(..), (:<)(..), (:\/:)(..), (:/\:)(..), (:||:)(..),
+                             FL(..), RL(..),
                              lengthFL, mapFL, mapFL_FL, spanFL, foldlFL, allFL, anyFL,
                              filterFL,
                              splitAtFL, splitAtRL, bunchFL, foldlRL,
                              lengthRL, isShorterThanRL, mapRL, mapRL_RL, zipWithFL,
-                             unsafeMap_l2f, filterE, filterFLFL,
+                             filterFLFL,
                              filterRL,
                              reverseFL, reverseRL, (+>+), (+<+),
-                             nullFL, concatFL, concatRL, concatReverseFL, headRL,
-                             MyEq, unsafeCompare, (=\/=), (=/\=),
+                             nullFL, concatFL, concatRL,
                              consRLSealed, nullRL, toFL,
-                             (:>>)(..),
-                             unsafeCoercePStart, unsafeCoercePEnd,
-                             unsafeCoerceP, unsafeCoerceP2, spanFL_M
+                             dropWhileFL, dropWhileRL,
+                             spanFL_M,
+                             eqFL, eqFLRev, eqFLUnsafe
                            ) where
 
 #include "impossible.h"
-import GHC.Base (unsafeCoerce#)
 import Darcs.Witnesses.Show
-import Darcs.Witnesses.Sealed ( FlippedSeal(..), flipSeal, Sealed(..), FreeLeft, unFreeLeft, Sealed2(..) )
-
-data EqCheck C(a b) where
-    IsEq :: EqCheck C(a a)
-    NotEq :: EqCheck C(a b)
-
-instance Eq (EqCheck C(a b)) where
-    IsEq == IsEq = True
-    NotEq == NotEq = True
-    _ == _ = False
-
-isEq :: EqCheck C(a b) -> Bool
-isEq IsEq = True
-isEq NotEq = False
-
-instance Show (EqCheck C(a b)) where
-    show IsEq = "IsEq"
-    show NotEq = "NotEq"
-
-data Proof a C(x y) where
-    Proof :: a -> Proof a C(x x)
+import Darcs.Witnesses.Sealed ( FlippedSeal(..), flipSeal, Sealed(..), FreeLeft, unFreeLeft, Sealed2(..),
+                                seal )
+import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
 
 data (a1 :> a2) C(x y) = FORALL(z) (a1 C(x z)) :> (a2 C(z y))
 infixr 1 :>
@@ -72,34 +50,6 @@ data (a1 :\/: a2) C(x y) = FORALL(z) (a1 C(z x)) :\/: (a2 C(z y))
 data (a1 :/\: a2) C(x y) = FORALL(z) (a1 C(x z)) :/\: (a2 C(y z))
 data (a1 :||: a2) C(x y) = (a1 C(x y)) :||: (a2 C(x y))
 
-data (a1 :>> a2) C(y) = FORALL(z) (a1 C(z)) :>> (a2 C(z y))
-infixr 1 :>>
-
-class MyEq p where
-    -- Minimal definition defines any one of unsafeCompare, =\/= and =/\=.
-    unsafeCompare :: p C(a b) -> p C(c d) -> Bool
-    unsafeCompare a b = IsEq == (a =/\= unsafeCoerceP b)
-    (=\/=) :: p C(a b) -> p C(a c) -> EqCheck C(b c)
-    a =\/= b | unsafeCompare a b = unsafeCoerceP IsEq
-             | otherwise = NotEq
-    (=/\=) :: p C(a c) -> p C(b c) -> EqCheck C(a b)
-    a =/\= b | IsEq == (a =\/= unsafeCoerceP b) = unsafeCoerceP IsEq
-             | otherwise = NotEq
-
-infix 4 =\/=, =/\=
-
-unsafeCoerceP :: a C(x y) -> a C(b c)
-unsafeCoerceP = unsafeCoerce#
-
-unsafeCoercePStart :: a C(x1 y) -> a C(x2 y)
-unsafeCoercePStart = unsafeCoerce#
-
-unsafeCoercePEnd :: a C(x y1) -> a C(x y2)
-unsafeCoercePEnd = unsafeCoerce#
-
-unsafeCoerceP2 :: t C(w x y z) -> t C(a b c d)
-unsafeCoerceP2 = unsafeCoerce#
-
 instance (Show2 a, Show2 b) => Show ( (a :> b) C(x y) ) where
     showsPrec d (x :> y) = showOp2 1 ":>" d x y
 
@@ -107,10 +57,16 @@ instance (MyEq a, MyEq b) => MyEq (a :> b) where
     (a1 :> b1) =\/= (a2 :> b2) | IsEq <- a1 =\/= a2 = b1 =\/= b2
                                | otherwise = NotEq
 
+instance (MyEq a, MyEq b) => Eq ((a :> b) C(x y)) where
+    (==) = unsafeCompare
+
 instance (MyEq a, MyEq b) => MyEq (a :< b) where
     (a1 :< b1) =\/= (a2 :< b2) | IsEq <- b1 =\/= b2 = a1 =\/= a2
                                | otherwise = NotEq
 
+instance (MyEq a, MyEq b) => Eq ((a :< b) C(x y)) where
+    (==) = unsafeCompare
+
 instance (Show2 a, Show2 b) => Show2 (a :> b) where
     showDict2 = ShowDictClass
 
@@ -120,10 +76,6 @@ instance (Show2 a, Show2 b) => Show ( (a :\/: b) C(x y) ) where
 instance (Show2 a, Show2 b) => Show2 (a :\/: b) where
     showDict2 = ShowDictClass
 
-instance MyEq a => Eq (Sealed (a C(x))) where
-    Sealed x == Sealed y | IsEq <- x =\/= y = True
-                         | otherwise = False
-
 infixr 5 :>:, :<:, +>+, +<+
 
 -- forward list
@@ -137,9 +89,27 @@ instance Show2 a => Show (FL a C(x z)) where
                             showString " :>: " . showsPrec (prec + 1) xs
        where prec = 5
 
+instance Show2 a => Show1 (FL a C(x)) where
+   showDict1 = ShowDictClass
+
 instance Show2 a => Show2 (FL a) where
    showDict2 = ShowDictClass
 
+instance Show2 a => Show (RL a C(x z)) where
+   showsPrec _ NilRL = showString "NilRL"
+   showsPrec d (x :<: xs) = showParen (d > prec) $ showsPrec2 (prec + 1) x .
+                            showString " :<: " . showsPrec (prec + 1) xs
+       where prec = 5
+
+instance Show2 a => Show1 (RL a C(x)) where
+   showDict1 = ShowDictClass
+
+instance Show2 a => Show2 (RL a) where
+   showDict2 = ShowDictClass
+
+instance (Show2 a, Show2 b) => Show1 ((a :> b) C(x)) where
+   showDict1 = ShowDictClass
+
 -- reverse list
 data RL a C(x z) where
     (:<:) :: a C(y z) -> RL a C(x y) -> RL a C(x z)
@@ -158,17 +128,11 @@ filterFLFL _ NilFL = NilFL
 filterFLFL f (x:>:xs) | IsEq <- f x = filterFLFL f xs
                     | otherwise = x :>: filterFLFL f xs
 
-filterRL :: (FORALL(x y) p C(x y) -> Bool) -> RL p C(x y) ->  [Sealed2 p]
+filterRL :: (FORALL(x y) p C(x y) -> Bool) -> RL p C(a b) ->  [Sealed2 p]
 filterRL _ NilRL = []
 filterRL f (x :<: xs) | f x = Sealed2 x : (filterRL f xs)
                       | otherwise = filterRL f xs
 
-filterE :: (a -> EqCheck C(x y)) -> [a] -> [Proof a C(x y)]
-filterE _ [] = []
-filterE p (x:xs)
-    | IsEq <- p x = Proof x : filterE p xs
-    | otherwise   = filterE p xs
-
 (+>+) :: FL a C(x y) -> FL a C(y z) -> FL a C(x z)
 NilFL +>+ ys = ys
 (x:>:xs) +>+ ys = x :>: xs +>+ ys
@@ -272,7 +236,7 @@ mapFL :: (FORALL(w z) a C(w z) -> b) -> FL a C(x y) -> [b]
 mapFL _ NilFL = []
 mapFL f (a :>: b) = f a : mapFL f b
 
-filterFL :: (FORALL(x y) a C(x y) -> Bool) -> FL a C(x y) -> [Sealed2 a]
+filterFL :: (FORALL(x y) a C(x y) -> Bool) -> FL a C(w z) -> [Sealed2 a]
 filterFL _ NilFL = []
 filterFL f (a :>: b) = if f a
                        then (Sealed2 a):(filterFL f b)
@@ -282,10 +246,6 @@ mapRL :: (FORALL(w z) a C(w z) -> b) -> RL a C(x y) -> [b]
 mapRL _ NilRL = []
 mapRL f (a :<: b) = f a : mapRL f b
 
-unsafeMap_l2f :: (FORALL(w z) a -> b C(w z)) -> [a] -> FL b C(x y)
-unsafeMap_l2f _ [] = unsafeCoerceP NilFL
-unsafeMap_l2f f (x:xs) = f x :>: unsafeMap_l2f f xs
-
 lengthFL :: FL a C(x z) -> Int
 lengthFL xs = l xs 0
   where l :: FL a C(x z) -> Int -> Int
@@ -303,13 +263,6 @@ isShorterThanRL _ n | n <= 0 = False
 isShorterThanRL NilRL _ = True
 isShorterThanRL (_:<:xs) n = isShorterThanRL xs (n-1)
 
-concatReverseFL :: FL (RL a) C(x y) -> RL a C(x y)
-concatReverseFL = concatRL . reverseFL
-
-headRL :: RL a C(x y) -> FlippedSeal a C(y)
-headRL (x:<:_) = flipSeal x
-headRL _ = impossible
-
 consRLSealed :: a C(y z) -> FlippedSeal (RL a) C(y) -> FlippedSeal (RL a) C(z)
 consRLSealed a (FlippedSeal as) = flipSeal $ a :<: as
 
@@ -317,3 +270,32 @@ toFL :: [FreeLeft a] -> Sealed (FL a C(x))
 toFL [] = Sealed NilFL
 toFL (x:xs) = case unFreeLeft x of Sealed y -> case toFL xs of Sealed ys -> Sealed (y :>: ys)
 
+dropWhileFL :: (FORALL(x y) a C(x y) -> Bool) -> FL a C(r v) -> FlippedSeal (FL a) C(v)
+dropWhileFL _ NilFL       = flipSeal NilFL
+dropWhileFL p xs@(x:>:xs')
+          | p x       = dropWhileFL p xs'
+          | otherwise = flipSeal xs
+
+dropWhileRL :: (FORALL(x y) a C(x y) -> Bool) -> RL a C(r v) -> Sealed (RL a C(r))
+dropWhileRL _ NilRL = seal NilRL
+dropWhileRL p xs@(x:<:xs')
+          | p x       = dropWhileRL p xs'
+          | otherwise = seal xs
+
+-- |Check that two 'FL's are equal element by element.
+-- This differs from the 'MyEq' instance for 'FL' which
+-- uses commutation.
+eqFL :: MyEq a => FL a C(x y) -> FL a C(x z) -> EqCheck C(y z)
+eqFL NilFL NilFL = IsEq
+eqFL (x:>:xs) (y:>:ys) | IsEq <- x =\/= y, IsEq <- eqFL xs ys = IsEq
+eqFL _ _ = NotEq
+
+eqFLRev :: MyEq a => FL a C(x z) -> FL a C(y z) -> EqCheck C(x y)
+eqFLRev NilFL NilFL = IsEq
+eqFLRev (x:>:xs) (y:>:ys) | IsEq <- eqFLRev xs ys, IsEq <- x =/\= y = IsEq
+eqFLRev _ _ = NotEq
+
+eqFLUnsafe :: MyEq a => FL a C(x y) -> FL a C(z w) -> Bool
+eqFLUnsafe NilFL NilFL = True
+eqFLUnsafe (x:>:xs) (y:>:ys) = unsafeCompare x y && eqFLUnsafe xs ys
+eqFLUnsafe _ _ = False
