@@ -8,7 +8,7 @@ rm -rf temp1 temp2
 mkdir temp1 temp2
 cd temp1
 darcs init
-cp ../$binary .
+cp $TESTDATA/$binary .
 darcs add $binary
 darcs record -am P1
 cd ../temp2
