@@ -14,12 +14,12 @@ darcs annotate -p .
 echo newtext > foo
 darcs record -am 'modify foo'
 
-darcs diff --store-in-mem --last=1 > out1
+darcs diff --no-unified --store-in-mem --last=1 > out1
 cat out1
 grep text out1
 grep foo out1
 
-darcs diff --last=1 > out
+darcs diff --no-unified --last=1 > out
 cat out
 grep text out
 grep foo out
