@@ -7,7 +7,7 @@ cd temp
 darcs init
 echo hi world > temp.c
 darcs add temp.c
-darcs record --all -A test --patch-name=hi
+darcs record --all -A test --name=hi
 echo goodbye >> temp.c
 darcs whatsnew
 darcs record -a -A au -m bye
